searchData={"items":[{"type":"module","title":"Hermes.Client","doc":"A GenServer implementation of an MCP (Model Context Protocol) client.\n\nThis module handles the client-side implementation of the MCP protocol,\nincluding initialization, request/response handling, and maintaining\nprotocol state.","ref":"Hermes.Client.html"},{"type":"module","title":"Examples - Hermes.Client","doc":"# Start a client process\n    {:ok, client} = Hermes.Client.start_link(\n      name: MyApp.MCPClient,\n      transport: Hermes.Transport.STDIO,\n      client_info: %{\"name\" => \"MyApp\", \"version\" => \"1.0.0\"},\n      capabilities: %{\"resources\" => %{}, \"tools\" => %{}}\n    )\n\n    # List available resources\n    {:ok, resources} = Hermes.Client.list_resources(client)","ref":"Hermes.Client.html#module-examples"},{"type":"module","title":"Notes - Hermes.Client","doc":"The initial client <> server handshake is performed automatically when the client is started.","ref":"Hermes.Client.html#module-notes"},{"type":"function","title":"Hermes.Client.call_tool/4","doc":"Calls a tool on the server.","ref":"Hermes.Client.html#call_tool/4"},{"type":"function","title":"Options - Hermes.Client.call_tool/4","doc":"* `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.html#call_tool/4-options"},{"type":"function","title":"Hermes.Client.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Client.html#child_spec/1"},{"type":"function","title":"Hermes.Client.close/1","doc":"Closes the client connection and terminates the process.","ref":"Hermes.Client.html#close/1"},{"type":"function","title":"Hermes.Client.get_prompt/4","doc":"Gets a specific prompt from the server.","ref":"Hermes.Client.html#get_prompt/4"},{"type":"function","title":"Options - Hermes.Client.get_prompt/4","doc":"* `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.html#get_prompt/4-options"},{"type":"function","title":"Hermes.Client.get_schema/1","doc":"","ref":"Hermes.Client.html#get_schema/1"},{"type":"function","title":"Hermes.Client.get_server_capabilities/1","doc":"Gets the server's capabilities as reported during initialization.\n\nReturns `nil` if the client has not been initialized yet.","ref":"Hermes.Client.html#get_server_capabilities/1"},{"type":"function","title":"Hermes.Client.get_server_info/1","doc":"Gets the server's information as reported during initialization.\n\nReturns `nil` if the client has not been initialized yet.","ref":"Hermes.Client.html#get_server_info/1"},{"type":"function","title":"Hermes.Client.list_prompts/2","doc":"Lists available prompts from the server.","ref":"Hermes.Client.html#list_prompts/2"},{"type":"function","title":"Options - Hermes.Client.list_prompts/2","doc":"* `:cursor` - Pagination cursor for continuing a previous request\n  * `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.html#list_prompts/2-options"},{"type":"function","title":"Hermes.Client.list_resources/2","doc":"Lists available resources from the server.","ref":"Hermes.Client.html#list_resources/2"},{"type":"function","title":"Options - Hermes.Client.list_resources/2","doc":"* `:cursor` - Pagination cursor for continuing a previous request\n  * `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.html#list_resources/2-options"},{"type":"function","title":"Hermes.Client.list_tools/2","doc":"Lists available tools from the server.","ref":"Hermes.Client.html#list_tools/2"},{"type":"function","title":"Options - Hermes.Client.list_tools/2","doc":"* `:cursor` - Pagination cursor for continuing a previous request\n  * `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.html#list_tools/2-options"},{"type":"function","title":"Hermes.Client.merge_capabilities/2","doc":"Merges additional capabilities into the client's capabilities.","ref":"Hermes.Client.html#merge_capabilities/2"},{"type":"type","title":"Hermes.Client.option/0","doc":"","ref":"Hermes.Client.html#t:option/0"},{"type":"function","title":"Hermes.Client.parse_options/1","doc":"","ref":"Hermes.Client.html#parse_options/1"},{"type":"function","title":"Hermes.Client.parse_options!/1","doc":"","ref":"Hermes.Client.html#parse_options!/1"},{"type":"function","title":"Hermes.Client.ping/2","doc":"Sends a ping request to the server to check connection health. Returns `:pong` if successful.","ref":"Hermes.Client.html#ping/2"},{"type":"function","title":"Hermes.Client.read_resource/3","doc":"Reads a specific resource from the server.","ref":"Hermes.Client.html#read_resource/3"},{"type":"function","title":"Options - Hermes.Client.read_resource/3","doc":"* `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.html#read_resource/3-options"},{"type":"function","title":"Hermes.Client.start_link/1","doc":"Starts a new MCP client process.","ref":"Hermes.Client.html#start_link/1"},{"type":"function","title":"Options - Hermes.Client.start_link/1","doc":"* `:name` - Optional name to register the client process\n  * `:transport` - The transport process or name to use (required)\n  * `:client_info` - Information about the client (required)\n  * `:capabilities` - Client capabilities to advertise\n  * `:protocol_version` - Protocol version to use (defaults to \"2024-11-05\")\n  * `:request_timeout` - Default timeout for requests in milliseconds (default: 30s)","ref":"Hermes.Client.html#start_link/1-options"},{"type":"module","title":"Hermes.HTTP","doc":"HTTP utilities.","ref":"Hermes.HTTP.html"},{"type":"function","title":"Hermes.HTTP.build/5","doc":"","ref":"Hermes.HTTP.html#build/5"},{"type":"function","title":"Hermes.HTTP.follow_redirect/2","doc":"","ref":"Hermes.HTTP.html#follow_redirect/2"},{"type":"function","title":"Hermes.HTTP.post/3","doc":"Performs a POST request to the given URL.","ref":"Hermes.HTTP.html#post/3"},{"type":"module","title":"Hermes.Message","doc":"Handles parsing and validation of MCP (Model Context Protocol) messages using the Peri library.\n\nThis module provides functions to parse and validate MCP messages based on the Model Context Protocol schema","ref":"Hermes.Message.html"},{"type":"function","title":"Hermes.Message.decode/1","doc":"Decodes raw data (possibly containing multiple messages) into JSON-RPC messages.\n\nReturns either:\n- `{:ok, messages}` where messages is a list of parsed JSON-RPC messages\n- `{:error, reason}` if parsing fails","ref":"Hermes.Message.html#decode/1"},{"type":"function","title":"Hermes.Message.encode_notification/1","doc":"Encodes a notification message to a JSON-RPC 2.0 compliant string.\n\nReturns the encoded string with a newline character appended.","ref":"Hermes.Message.html#encode_notification/1"},{"type":"function","title":"Hermes.Message.encode_request/2","doc":"Encodes a request message to a JSON-RPC 2.0 compliant string.\n\nReturns the encoded string with a newline character appended.","ref":"Hermes.Message.html#encode_request/2"},{"type":"function","title":"Hermes.Message.encode_response/2","doc":"Encodes a response message to a JSON-RPC 2.0 compliant string.\n\nReturns the encoded string with a newline character appended.","ref":"Hermes.Message.html#encode_response/2"},{"type":"function","title":"Hermes.Message.error_schema/1","doc":"","ref":"Hermes.Message.html#error_schema/1"},{"type":"function","title":"Hermes.Message.error_schema!/1","doc":"","ref":"Hermes.Message.html#error_schema!/1"},{"type":"function","title":"Hermes.Message.get_schema/1","doc":"","ref":"Hermes.Message.html#get_schema/1"},{"type":"macro","title":"Hermes.Message.is_error/1","doc":"Determines if a JSON-RPC message is an error.","ref":"Hermes.Message.html#is_error/1"},{"type":"macro","title":"Hermes.Message.is_notification/1","doc":"Determines if a JSON-RPC message is a notification.","ref":"Hermes.Message.html#is_notification/1"},{"type":"macro","title":"Hermes.Message.is_request/1","doc":"Determines if a JSON-RPC message is a request.","ref":"Hermes.Message.html#is_request/1"},{"type":"macro","title":"Hermes.Message.is_response/1","doc":"Determines if a JSON-RPC message is a response.","ref":"Hermes.Message.html#is_response/1"},{"type":"function","title":"Hermes.Message.mcp_message_schema/1","doc":"","ref":"Hermes.Message.html#mcp_message_schema/1"},{"type":"function","title":"Hermes.Message.mcp_message_schema!/1","doc":"","ref":"Hermes.Message.html#mcp_message_schema!/1"},{"type":"function","title":"Hermes.Message.notification_schema/1","doc":"","ref":"Hermes.Message.html#notification_schema/1"},{"type":"function","title":"Hermes.Message.notification_schema!/1","doc":"","ref":"Hermes.Message.html#notification_schema!/1"},{"type":"function","title":"Hermes.Message.request_schema/1","doc":"","ref":"Hermes.Message.html#request_schema/1"},{"type":"function","title":"Hermes.Message.request_schema!/1","doc":"","ref":"Hermes.Message.html#request_schema!/1"},{"type":"function","title":"Hermes.Message.response_schema/1","doc":"","ref":"Hermes.Message.html#response_schema/1"},{"type":"function","title":"Hermes.Message.response_schema!/1","doc":"","ref":"Hermes.Message.html#response_schema!/1"},{"type":"function","title":"Hermes.Message.validate_message/1","doc":"Validates a decoded JSON message to ensure it complies with the MCP schema.","ref":"Hermes.Message.html#validate_message/1"},{"type":"module","title":"Hermes.SSE","doc":"Server-Sent Events (SSE) utilities, HTTP client abstraction and HTTP streaming handling.","ref":"Hermes.SSE.html"},{"type":"function","title":"Hermes.SSE.connect/3","doc":"Connects to a server-sent event stream.","ref":"Hermes.SSE.html#connect/3"},{"type":"function","title":"Parameters - Hermes.SSE.connect/3","doc":"- `server_url` - the URL of the server to connect to.\n  - `headers` - additional headers to send with the request.\n  - `opts` - additional options to pass to the HTTP client.","ref":"Hermes.SSE.html#connect/3-parameters"},{"type":"function","title":"Examples - Hermes.SSE.connect/3","doc":"iex> Hermes.SSE.connect(\"http://localhost:4000\")\n    #Stream<[ref: 1, task: #PID<0.123.0>]>","ref":"Hermes.SSE.html#connect/3-examples"},{"type":"module","title":"Hermes.SSE.Event","doc":"Represents a Server-Sent Event.\n\nFields:\n  - `:id`    - identifier of the event (if any)\n  - `:event` - event type (defaults to \"message\")\n  - `:data`  - the event data (concatenates multiple data lines with a newline)\n  - `:retry` - reconnection time (parsed as integer, if provided)","ref":"Hermes.SSE.Event.html"},{"type":"type","title":"Hermes.SSE.Event.t/0","doc":"","ref":"Hermes.SSE.Event.html#t:t/0"},{"type":"module","title":"Hermes.SSE.Parser","doc":"Parses a raw SSE stream into a list of `%SSE.Event{}` structs.","ref":"Hermes.SSE.Parser.html"},{"type":"function","title":"Hermes.SSE.Parser.run/1","doc":"Parses a string containing one or more SSE events.\n\nEach event is separated by an empty line (two consecutive newlines).\nReturns a list of `%SSE.Event{}` structs.","ref":"Hermes.SSE.Parser.html#run/1"},{"type":"behaviour","title":"Hermes.Transport.Behaviour","doc":"Defines the behavior that all transport implementations must follow.","ref":"Hermes.Transport.Behaviour.html"},{"type":"type","title":"Hermes.Transport.Behaviour.message/0","doc":"","ref":"Hermes.Transport.Behaviour.html#t:message/0"},{"type":"type","title":"Hermes.Transport.Behaviour.reason/0","doc":"","ref":"Hermes.Transport.Behaviour.html#t:reason/0"},{"type":"callback","title":"Hermes.Transport.Behaviour.send_message/1","doc":"","ref":"Hermes.Transport.Behaviour.html#c:send_message/1"},{"type":"callback","title":"Hermes.Transport.Behaviour.send_message/2","doc":"","ref":"Hermes.Transport.Behaviour.html#c:send_message/2"},{"type":"callback","title":"Hermes.Transport.Behaviour.shutdown/1","doc":"","ref":"Hermes.Transport.Behaviour.html#c:shutdown/1"},{"type":"callback","title":"Hermes.Transport.Behaviour.start_link/1","doc":"","ref":"Hermes.Transport.Behaviour.html#c:start_link/1"},{"type":"type","title":"Hermes.Transport.Behaviour.t/0","doc":"","ref":"Hermes.Transport.Behaviour.html#t:t/0"},{"type":"module","title":"Hermes.Transport.SSE","doc":"A transport implementation that uses Server-Sent Events (SSE) for receiving messages\nand HTTP POST requests for sending messages back to the server.","ref":"Hermes.Transport.SSE.html"},{"type":"module","title":"Examples - Hermes.Transport.SSE","doc":"iex> Hermes.Transport.SSE.start_link(server_url: \"http://localhost:4000\")\n    {:ok, #PID<0.123.0>}\n\n    iex> Hermes.Transport.SSE.send_message(pid, \"Hello, world!\")","ref":"Hermes.Transport.SSE.html#module-examples"},{"type":"module","title":"Options - Hermes.Transport.SSE","doc":"The options schema for the SSE transport.\n\n- `name`: The name of the transport.\n- `client`: The client process to send messages to.\n- `server_url`: The URL of the server to connect to.\n- `headers`: Additional headers to send with the HTTP requests.\n- `transport_opts`: Additional options as keyword to pass to the HTTP client, you can reference the available options at https://hexdocs.pm/mint/Mint.HTTP.html#connect/4-transport-options\n- `http_options`: Additional HTTP request options as keyword to pass to the HTTP client, you can check the available options on https://hexdocs.pm/finch/Finch.html#t:request_opt/0","ref":"Hermes.Transport.SSE.html#module-options"},{"type":"function","title":"Hermes.Transport.SSE.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Transport.SSE.html#child_spec/1"},{"type":"function","title":"Hermes.Transport.SSE.get_schema/1","doc":"","ref":"Hermes.Transport.SSE.html#get_schema/1"},{"type":"type","title":"Hermes.Transport.SSE.option/0","doc":"","ref":"Hermes.Transport.SSE.html#t:option/0"},{"type":"function","title":"Hermes.Transport.SSE.options_schema/1","doc":"","ref":"Hermes.Transport.SSE.html#options_schema/1"},{"type":"function","title":"Hermes.Transport.SSE.options_schema!/1","doc":"","ref":"Hermes.Transport.SSE.html#options_schema!/1"},{"type":"type","title":"Hermes.Transport.SSE.params_t/0","doc":"","ref":"Hermes.Transport.SSE.html#t:params_t/0"},{"type":"module","title":"Hermes.Transport.STDIO","doc":"A transport implementation that uses standard input/output.","ref":"Hermes.Transport.STDIO.html"},{"type":"function","title":"Hermes.Transport.STDIO.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Transport.STDIO.html#child_spec/1"},{"type":"function","title":"Hermes.Transport.STDIO.get_schema/1","doc":"","ref":"Hermes.Transport.STDIO.html#get_schema/1"},{"type":"type","title":"Hermes.Transport.STDIO.option/0","doc":"","ref":"Hermes.Transport.STDIO.html#t:option/0"},{"type":"function","title":"Hermes.Transport.STDIO.options_schema/1","doc":"","ref":"Hermes.Transport.STDIO.html#options_schema/1"},{"type":"function","title":"Hermes.Transport.STDIO.options_schema!/1","doc":"","ref":"Hermes.Transport.STDIO.html#options_schema!/1"},{"type":"type","title":"Hermes.Transport.STDIO.params_t/0","doc":"","ref":"Hermes.Transport.STDIO.html#t:params_t/0"},{"type":"task","title":"mix sse.interactive","doc":"Mix task to test the SSE transport implementation, interactively sending commands.","ref":"Mix.Tasks.Sse.Interactive.html"},{"type":"function","title":"Mix.Tasks.Sse.Interactive.run/1","doc":"","ref":"Mix.Tasks.Sse.Interactive.html#run/1"},{"type":"task","title":"mix stdio","doc":"Mix task to test the STDIO transport implementation.","ref":"Mix.Tasks.Stdio.html"},{"type":"function","title":"Mix.Tasks.Stdio.run/1","doc":"","ref":"Mix.Tasks.Stdio.html#run/1"},{"type":"task","title":"mix stdio.interactive","doc":"Mix task to test the STDIO transport implementation, interactively sending commands.","ref":"Mix.Tasks.Stdio.Interactive.html"},{"type":"function","title":"Mix.Tasks.Stdio.Interactive.loop/1","doc":"","ref":"Mix.Tasks.Stdio.Interactive.html#loop/1"},{"type":"function","title":"Mix.Tasks.Stdio.Interactive.run/1","doc":"","ref":"Mix.Tasks.Stdio.Interactive.html#run/1"},{"type":"extras","title":"Hermes MCP","doc":"# Hermes MCP\n\n> [!WARNING]\n>\n> This library is under active development, may expect breaking changes\n\nA high-performance Model Context Protocol (MCP) implementation in Elixir with first-class Phoenix support.","ref":"readme.html"},{"type":"extras","title":"Overview - Hermes MCP","doc":"Hermes MCP provides a unified solution for building both MCP clients and servers in Elixir, leveraging the language's exceptional concurrency model and fault tolerance capabilities. The library offers seamless integration with Phoenix for HTTP/SSE transport while maintaining support for standard stdio communication.","ref":"readme.html#overview"},{"type":"extras","title":"Features - Hermes MCP","doc":"The library implements the full [Model Context Protocol specification](https://spec.modelcontextprotocol.io/specification/2024-11-05/), providing:\n\n- Complete client and server implementations with protocol lifecycle management \n- First-class Phoenix integration for HTTP/SSE transport\n- Robust stdio transport for local process communication\n- Built-in connection supervision and automatic recovery\n- Comprehensive capability negotiation","ref":"readme.html#features"},{"type":"extras","title":"Installation - Hermes MCP","doc":"Add Hermes MCP to your dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:hermes_mcp, \"~> 0.2\"}\n  ]\nend\n```\n\nBased on the provided documents, I'll create a section for the README that demonstrates a minimal usage example of the Hermes client interface and transport with supervision tree integration.","ref":"readme.html#installation"},{"type":"extras","title":"Usage Example - Hermes MCP","doc":"","ref":"readme.html#usage-example"},{"type":"extras","title":"Creating a Supervised Client - Hermes MCP","doc":"The following example shows how to integrate Hermes MCP into your application's supervision tree for robust client management:\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      # Start the MCP transport\n      {Hermes.Transport.STDIO, [\n        name: MyApp.MCPTransport,\n        client: MyApp.MCPClient, \n        command: \"mcp\",\n        args: [\"run\", \"path/to/server.py\"]\n      ]},\n      \n      # Start the MCP client using the transport\n      {Hermes.Client, [\n        name: MyApp.MCPClient,\n        transport: MyApp.MCPTransport,\n        client_info: %{\n          \"name\" => \"MyApp\",\n          \"version\" => \"1.0.0\"\n        },\n        capabilities: %{\n          \"resources\" => %{},\n          \"tools\" => %{},\n          \"prompts\" => %{}\n        }\n      ]}\n      \n      # Your other application services\n      # ...\n    ]\n    \n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"readme.html#creating-a-supervised-client"},{"type":"extras","title":"Making Client Requests - Hermes MCP","doc":"Once your client is running, you can interact with the MCP server:\n\n```elixir\n# List available resources\n{:ok, %{\"resources\" => resources}} = Hermes.Client.list_resources(MyApp.MCPClient)\n\n# Read a specific resource\n{:ok, resource} = Hermes.Client.read_resource(MyApp.MCPClient, \"file:///example.txt\")\n\n# List available tools\n{:ok, %{\"tools\" => tools}} = Hermes.Client.list_tools(MyApp.MCPClient)\n\n# Call a tool\n{:ok, result} = Hermes.Client.call_tool(MyApp.MCPClient, \"example_tool\", %{\"param\" => \"value\"})\n  \n# List available prompts\n{:ok, %{\"prompts\" => prompts}} = Hermes.Client.list_prompts(MyApp.MCPClient)\n\n# Get a prompt with arguments\n{:ok, prompt} = Hermes.Client.get_prompt(MyApp.MCPClient, \"example_prompt\", %{\"arg\" => \"value\"})\n```","ref":"readme.html#making-client-requests"},{"type":"extras","title":"Logging - Hermes MCP","doc":"`hermes-mcp` follows the [Logger](https://hexdocs.pm/logger) standards and also provide additional metadata:\n- `mcp_client`: the current `Hermes.MCP.Client` process\n- `mcp_transport`: the current `Hermes.Transport` layer being used by the client, either `STDIO` or `SSE`","ref":"readme.html#logging"},{"type":"extras","title":"Error Handling - Hermes MCP","doc":"Hermes provides standardized error handling:\n\n```elixir\ncase Hermes.Client.call_tool(MyApp.MCPClient, \"unavailable_tool\", %{}) do\n  {:ok, result} ->\n    # Handle successful result\n    IO.inspect(result)\n    \n  {:error, error} ->\n    # Handle error response\n    IO.puts(\"Error: #{error[\"message\"]} (Code: #{error[\"code\"]})\")\nend\n```\n\nThe client automatically manages the connection lifecycle, including initial handshake, capability negotiation, and message correlation.","ref":"readme.html#error-handling"},{"type":"extras","title":"Architecture - Hermes MCP","doc":"The library is structured around several core components:\n\n- Protocol Layer: Handles message framing, request/response correlation, and lifecycle management\n- Transport Layer: Provides pluggable transports for stdio and HTTP/SSE communication\n- Server Components: Implements server-side protocol operations with Phoenix integration\n- Client Components: Manages client-side operations and capability negotiation\n- Supervision Trees: Ensures fault tolerance and automatic recovery\n\nCheck out our technical [RFC](./rfc.md) that describe each component more in deep.","ref":"readme.html#architecture"},{"type":"extras","title":"Development Status - Hermes MCP","doc":"Hermes MCP is currently under active development. The initial release will focus on providing stable protocol implementation focused into client interface to existing MCP servers. The API is subject to change as we refine the design based on real-world usage patterns.\n\nWe encourage you to star the repository and watch for updates as we progress toward our first stable release.","ref":"readme.html#development-status"},{"type":"extras","title":"Contributing - Hermes MCP","doc":"We welcome contributions to Hermes MCP! The project is in its early stages, and we're particularly interested in feedback on the design and architecture. Please check our [Issues](https://github.com/cloudwalk/hermes-mcp/issues) page for ways to contribute.","ref":"readme.html#contributing"},{"type":"extras","title":"License - Hermes MCP","doc":"Hermes MCP is released under the MIT License. See [LICENSE](LICENSE) for details.","ref":"readme.html#license"},{"type":"extras","title":"Why Hermes? - Hermes MCP","doc":"The library is named after Hermes, the Greek god of boundaries, communication, and commerce. This namesake reflects the core purpose of the Model Context Protocol: to establish standardized communication between AI applications and external tools. Like Hermes who served as a messenger between gods and mortals, this library facilitates seamless interaction between Large Language Models and various data sources or tools.\n\nFurthermore, Hermes was known for his speed and reliability in delivering messages, which aligns with our implementation's focus on high performance and fault tolerance in the Elixir ecosystem. The name also draws inspiration from the Hermetic tradition of bridging different domains of knowledge, much like how MCP bridges the gap between AI models and external capabilities.","ref":"readme.html#why-hermes"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project are documented in this file.\n\n## [0.1.0](https://github.com/cloudwalk/hermes-mcp) - 2025-02-26","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Implemented STDIO transport (#1) for MCP communication\n  - Support for bidirectional communication via standard I/O\n  - Automatic process monitoring and recovery\n  - Environment variable handling for cross-platform support\n  - Integration test utilities in Mix tasks\n\n- Created stateful client interface (#6)\n  - Robust GenServer implementation for MCP client\n  - Automatic initialization and protocol handshake\n  - Synchronous-feeling API over asynchronous transport\n  - Support for all MCP operations (ping, resources, prompts, tools)\n  - Proper error handling and logging\n  - Capability negotiation and management\n\n- Developed JSON-RPC message parsing (#5)\n  - Schema-based validation of MCP messages\n  - Support for requests, responses, notifications, and errors\n  - Comprehensive test suite for message handling\n  - Encoding/decoding functions with proper validation\n\n- Established core architecture and client API\n  - MCP protocol implementation following specification\n  - Client struct for maintaining connection state\n  - Request/response correlation with unique IDs\n  - Initial transport abstraction layer","ref":"changelog.html#added"},{"type":"extras","title":"Documentation - Changelog","doc":"- Added detailed RFC document describing the library architecture\n- Enhanced README with project overview and installation instructions","ref":"changelog.html#documentation"},{"type":"extras","title":"LICENSE","doc":"Copyright 2025 zoedsoupe  \n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"},{"type":"extras","title":"Hermes MCP","doc":"# Hermes MCP\n\nA high-performance Model Context Protocol (MCP) implementation in Elixir with first-class Phoenix support.","ref":"home.html"},{"type":"extras","title":"Overview - Hermes MCP","doc":"Hermes MCP provides a unified solution for building both MCP clients and servers in Elixir, leveraging the language's exceptional concurrency model and fault tolerance capabilities. The library currently focuses on a robust client implementation, with server functionality planned for future releases.","ref":"home.html#overview"},{"type":"extras","title":"Implementation Status - Hermes MCP","doc":"- ✅ Complete client implementation with protocol lifecycle management\n- ✅ STDIO transport for local process communication\n- ✅ Comprehensive capability negotiation\n- ✅ Robust error handling and recovery\n- 🔄 Phoenix integration (in progress)\n- 🔄 Full server implementation (planned)","ref":"home.html#implementation-status"},{"type":"extras","title":"Protocol Compliance - Hermes MCP","doc":"Hermes MCP implements the [Model Context Protocol specification](https://spec.modelcontextprotocol.io/specification/2024-11-05/), ensuring interoperability with other MCP-compliant tools and services. The library handles all aspects of the MCP lifecycle, from initialization and capability negotiation to request routing and response handling.","ref":"home.html#protocol-compliance"},{"type":"extras","title":"Why Hermes? - Hermes MCP","doc":"The library is named after Hermes, the Greek god of boundaries, communication, and commerce. This namesake reflects the core purpose of the Model Context Protocol: to establish standardized communication between AI applications and external tools. Like Hermes who served as a messenger between gods and mortals, this library facilitates seamless interaction between Large Language Models and various data sources or tools.\n\nFurthermore, Hermes was known for his speed and reliability in delivering messages, which aligns with our implementation's focus on high performance and fault tolerance in the Elixir ecosystem.","ref":"home.html#why-hermes"},{"type":"extras","title":"Installation & Setup","doc":"# Installation & Setup\n\nSetting up Hermes MCP for your Elixir project involves a few straightforward steps.","ref":"installation.html"},{"type":"extras","title":"Adding the Dependency - Installation & Setup","doc":"Add Hermes MCP to your dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:hermes_mcp, \"~> 0.2.0\"}\n  ]\nend\n```\n\nThen run:\n\n```shell\nmix deps.get\n```","ref":"installation.html#adding-the-dependency"},{"type":"extras","title":"Supervision Tree Integration - Installation & Setup","doc":"Hermes MCP is designed to be integrated into your application's supervision tree. This provides robust lifecycle management and automatic recovery in case of failures.","ref":"installation.html#supervision-tree-integration"},{"type":"extras","title":"Basic Integration Example - Installation & Setup","doc":"```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      # Start the MCP transport\n      {Hermes.Transport.STDIO, [\n        name: MyApp.MCPTransport,\n        client: MyApp.MCPClient, \n        command: \"mcp\",\n        args: [\"run\", \"path/to/server.py\"]\n      ]},\n      \n      # Start the MCP client using the transport\n      {Hermes.Client, [\n        name: MyApp.MCPClient,\n        transport: MyApp.MCPTransport,\n        client_info: %{\n          \"name\" => \"MyApp\",\n          \"version\" => \"1.0.0\"\n        },\n        capabilities: %{\n          \"resources\" => %{},\n          \"tools\" => %{},\n          \"prompts\" => %{}\n        }\n      ]}\n      \n      # Your other application services\n      # ...\n    ]\n    \n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"installation.html#basic-integration-example"},{"type":"extras","title":"Configuration Options - Installation & Setup","doc":"","ref":"installation.html#configuration-options"},{"type":"extras","title":"Client Configuration - Installation & Setup","doc":"The `Hermes.Client` accepts the following options:\n\n| Option | Type | Description | Default |\n|--------|------|-------------|---------|\n| `:name` | atom | Registration name for the client process | `__MODULE__` |\n| `:transport` | module or pid | The transport module or process | Required |\n| `:client_info` | map | Information about the client | Required |\n| `:capabilities` | map | Client capabilities to advertise | `%{\"resources\" => %{}, \"tools\" => %{}}` |\n| `:protocol_version` | string | Protocol version to use | `\"2024-11-05\"` |\n| `:request_timeout` | integer | Default timeout for requests in milliseconds | 30000 |","ref":"installation.html#client-configuration"},{"type":"extras","title":"Transport Configuration - Installation & Setup","doc":"The `Hermes.Transport.STDIO` accepts the following options:\n\n| Option | Type | Description | Default |\n|--------|------|-------------|---------|\n| `:name` | atom | Registration name for the transport process | `__MODULE__` |\n| `:client` | atom or pid | The client process that will receive messages | Required |\n| `:command` | string | The command to execute | Required |\n| `:args` | list | Command line arguments | `nil` |\n| `:env` | map | Environment variables | System defaults |\n| `:cwd` | string | Working directory | Current directory |","ref":"installation.html#transport-configuration"},{"type":"extras","title":"Environment Requirements - Installation & Setup","doc":"- Elixir 1.18 or later\n- Erlang/OTP 26 or later","ref":"installation.html#environment-requirements"},{"type":"extras","title":"Client Usage Guide","doc":"# Client Usage Guide\n\nThis guide covers the basic usage patterns for the Hermes MCP client.","ref":"client_usage.html"},{"type":"extras","title":"Client Lifecycle - Client Usage Guide","doc":"Once properly configured in your application's supervision tree, the Hermes client automatically handles:\n\n1. Initial connection to the server\n2. Protocol version negotiation\n3. Capability exchange\n4. Handshake completion","ref":"client_usage.html#client-lifecycle"},{"type":"extras","title":"Basic Operations - Client Usage Guide","doc":"","ref":"client_usage.html#basic-operations"},{"type":"extras","title":"Checking Connection Status - Client Usage Guide","doc":"You can check if a server is responsive with the `ping/1` function:\n\n```elixir\ncase Hermes.Client.ping(MyApp.MCPClient) do\n  :pong -> IO.puts(\"Server is responsive\")\n  {:error, reason} -> IO.puts(\"Connection error: #{inspect(reason)}\")\nend\n```","ref":"client_usage.html#checking-connection-status"},{"type":"extras","title":"Retrieving Server Information - Client Usage Guide","doc":"After initialization, you can access the server's capabilities and information:\n\n```elixir\n# Get server capabilities\nserver_capabilities = Hermes.Client.get_server_capabilities(MyApp.MCPClient)\nIO.inspect(server_capabilities, label: \"Server capabilities\")\n\n# Get server version information\nserver_info = Hermes.Client.get_server_info(MyApp.MCPClient)\nIO.inspect(server_info, label: \"Server info\")\n```","ref":"client_usage.html#retrieving-server-information"},{"type":"extras","title":"Working with Resources - Client Usage Guide","doc":"","ref":"client_usage.html#working-with-resources"},{"type":"extras","title":"Listing Resources - Client Usage Guide","doc":"To retrieve the list of available resources from the server:\n\n```elixir\ncase Hermes.Client.list_resources(MyApp.MCPClient) do\n  {:ok, %{\"resources\" => resources}} ->\n    IO.puts(\"Available resources:\")\n    Enum.each(resources, fn resource ->\n      IO.puts(\"  - #{resource[\"name\"]} (#{resource[\"uri\"]})\")\n    end)\n    \n  {:error, error} ->\n    IO.puts(\"Error listing resources: #{inspect(error)}\")\nend\n```","ref":"client_usage.html#listing-resources"},{"type":"extras","title":"Pagination - Client Usage Guide","doc":"For large resource collections, you can use pagination with cursors:\n\n```elixir\n# First page\n{:ok, %{\"resources\" => resources, \"nextCursor\" => cursor}} = \n  Hermes.Client.list_resources(MyApp.MCPClient)\n\n# Get next page if a cursor is available\nif cursor do\n  {:ok, %{\"resources\" => more_resources}} = \n    Hermes.Client.list_resources(MyApp.MCPClient, cursor: cursor)\nend\n```","ref":"client_usage.html#pagination"},{"type":"extras","title":"Reading Resources - Client Usage Guide","doc":"To read the contents of a specific resource:\n\n```elixir\ncase Hermes.Client.read_resource(MyApp.MCPClient, \"file:///example.txt\") do\n  {:ok, %{\"contents\" => contents}} ->\n    Enum.each(contents, fn content ->\n      case content do\n        %{\"text\" => text} -> IO.puts(\"Text content: #{text}\")\n        %{\"blob\" => blob} -> IO.puts(\"Binary content: #{byte_size(blob)} bytes\")\n      end\n    end)\n    \n  {:error, error} ->\n    IO.puts(\"Error reading resource: #{inspect(error)}\")\nend\n```","ref":"client_usage.html#reading-resources"},{"type":"extras","title":"Working with Tools - Client Usage Guide","doc":"","ref":"client_usage.html#working-with-tools"},{"type":"extras","title":"Listing Tools - Client Usage Guide","doc":"To discover available tools:\n\n```elixir\ncase Hermes.Client.list_tools(MyApp.MCPClient) do\n  {:ok, %{\"tools\" => tools}} ->\n    IO.puts(\"Available tools:\")\n    Enum.each(tools, fn tool ->\n      IO.puts(\"  - #{tool[\"name\"]}: #{tool[\"description\"] || \"No description\"}\")\n    end)\n    \n  {:error, error} ->\n    IO.puts(\"Error listing tools: #{inspect(error)}\")\nend\n```","ref":"client_usage.html#listing-tools"},{"type":"extras","title":"Calling Tools - Client Usage Guide","doc":"To invoke a tool with arguments:\n\n```elixir\ntool_name = \"calculate\"\ntool_args = %{\"expression\" => \"2 + 2\"}\n\ncase Hermes.Client.call_tool(MyApp.MCPClient, tool_name, tool_args) do\n  {:ok, %{\"content\" => content, \"isError\" => false}} ->\n    IO.puts(\"Tool result:\")\n    Enum.each(content, fn item ->\n      case item do\n        %{\"type\" => \"text\", \"text\" => text} -> IO.puts(\"  #{text}\")\n        _ -> IO.puts(\"  #{inspect(item)}\")\n      end\n    end)\n    \n  {:ok, %{\"content\" => content, \"isError\" => true}} ->\n    IO.puts(\"Tool execution error:\")\n    Enum.each(content, fn item ->\n      case item do\n        %{\"type\" => \"text\", \"text\" => text} -> IO.puts(\"  #{text}\")\n        _ -> IO.puts(\"  #{inspect(item)}\")\n      end\n    end)\n    \n  {:error, error} ->\n    IO.puts(\"Error calling tool: #{inspect(error)}\")\nend\n```","ref":"client_usage.html#calling-tools"},{"type":"extras","title":"Working with Prompts - Client Usage Guide","doc":"","ref":"client_usage.html#working-with-prompts"},{"type":"extras","title":"Listing Prompts - Client Usage Guide","doc":"To list available prompts:\n\n```elixir\ncase Hermes.Client.list_prompts(MyApp.MCPClient) do\n  {:ok, %{\"prompts\" => prompts}} ->\n    IO.puts(\"Available prompts:\")\n    Enum.each(prompts, fn prompt ->\n      required_args = prompt[\"arguments\"] \n                      |> Enum.filter(& &1[\"required\"])\n                      |> Enum.map(& &1[\"name\"])\n                      |> Enum.join(\", \")\n      \n      IO.puts(\"  - #{prompt[\"name\"]}\")\n      IO.puts(\"    Required args: #{required_args}\")\n    end)\n    \n  {:error, error} ->\n    IO.puts(\"Error listing prompts: #{inspect(error)}\")\nend\n```","ref":"client_usage.html#listing-prompts"},{"type":"extras","title":"Getting Prompts - Client Usage Guide","doc":"To retrieve a specific prompt with arguments:\n\n```elixir\nprompt_name = \"code_review\"\nprompt_args = %{\"code\" => \"def hello():\\n    print('world')\"}\n\ncase Hermes.Client.get_prompt(MyApp.MCPClient, prompt_name, prompt_args) do\n  {:ok, %{\"messages\" => messages}} ->\n    IO.puts(\"Prompt messages:\")\n    Enum.each(messages, fn message ->\n      role = message[\"role\"]\n      content = case message[\"content\"] do\n        %{\"type\" => \"text\", \"text\" => text} -> text\n        _ -> inspect(message[\"content\"])\n      end\n      \n      IO.puts(\"  #{role}: #{content}\")\n    end)\n    \n  {:error, error} ->\n    IO.puts(\"Error getting prompt: #{inspect(error)}\")\nend\n```","ref":"client_usage.html#getting-prompts"},{"type":"extras","title":"Error Handling - Client Usage Guide","doc":"Hermes follows a consistent pattern for error handling:\n\n```elixir\ncase Hermes.Client.some_operation(MyApp.MCPClient, args) do\n  {:ok, result} ->\n    # Handle successful result\n    \n  {:error, %{\"code\" => code, \"message\" => message}} ->\n    # Handle protocol-level error\n    IO.puts(\"Protocol error #{code}: #{message}\")\n    \n  {:error, {:transport_error, reason}} ->\n    # Handle transport-level error\n    IO.puts(\"Transport error: #{inspect(reason)}\")\n    \n  {:error, other_error} ->\n    # Handle other errors\n    IO.puts(\"Unexpected error: #{inspect(other_error)}\")\nend\n```","ref":"client_usage.html#error-handling"},{"type":"extras","title":"Timeouts and Cancellation - Client Usage Guide","doc":"You can specify custom timeouts for operations:\n\n```elixir\n# Use a custom timeout (in milliseconds)\nHermes.Client.call_tool(MyApp.MCPClient, \"slow_tool\", %{}, timeout: 60_000)\n```","ref":"client_usage.html#timeouts-and-cancellation"},{"type":"extras","title":"Extended Capabilities - Client Usage Guide","doc":"To extend the client's capabilities after initialization:\n\n```elixir\n# Add sampling capability\nnew_capabilities = %{\"sampling\" => %{}}\nupdated_capabilities = Hermes.Client.merge_capabilities(MyApp.MCPClient, new_capabilities)\n```","ref":"client_usage.html#extended-capabilities"},{"type":"extras","title":"Graceful Shutdown - Client Usage Guide","doc":"To gracefully close a client connection:\n\n```elixir\nHermes.Client.close(MyApp.MCPClient)\n```","ref":"client_usage.html#graceful-shutdown"},{"type":"extras","title":"Transport Options","doc":"# Transport Options\n\nHermes MCP supports multiple transport mechanisms to connect clients with servers. This page covers the available options and how to configure them.","ref":"transport_options.html"},{"type":"extras","title":"Transport Behavior - Transport Options","doc":"All transports in Hermes implement the `Hermes.Transport.Behaviour` behavior, which defines the interface for sending and receiving messages:\n\n```elixir\ndefmodule Hermes.Transport.Behaviour do\n  @type t :: pid() | atom()\n\n  @typedoc \"encoded JSON-RPC message\"\n  @type message :: String.t()\n\n  @callback start_link(keyword()) :: Supervisor.on_start()\n  @callback send_message(message()) :: :ok | {:error, reason()}\n  @callback send_message(t(), message()) :: :ok | {:error, reason()}\n  @callback shutdown(t()) :: :ok | {:error, reason()}\n  \n  @optional_callbacks send_message: 1\nend\n```\n\nThis standardized interface allows the client to work with different transport mechanisms interchangeably.","ref":"transport_options.html#transport-behavior"},{"type":"extras","title":"STDIO Transport - Transport Options","doc":"The STDIO transport (`Hermes.Transport.STDIO`) enables communication with an MCP server running as a separate process. It's suitable for local integrations and communicates through standard input/output streams.","ref":"transport_options.html#stdio-transport"},{"type":"extras","title":"Configuration - Transport Options","doc":"```elixir\n{Hermes.Transport.STDIO, [\n  name: MyApp.MCPTransport,\n  client: MyApp.MCPClient, \n  command: \"mcp\",\n  args: [\"run\", \"path/to/server.py\"],\n  env: %{\"CUSTOM_VAR\" => \"value\"},\n  cwd: \"/path/to/working/directory\"\n]}\n```","ref":"transport_options.html#configuration"},{"type":"extras","title":"Configuration Options - Transport Options","doc":"| Option | Type | Description | Default |\n|--------|------|-------------|---------|\n| `:name` | atom | Registration name for the transport process | `__MODULE__` |\n| `:client` | atom or pid | The client process that will receive messages | Required |\n| `:command` | string | The command to execute | Required |\n| `:args` | list | Command line arguments | `nil` |\n| `:env` | map | Environment variables | System defaults |\n| `:cwd` | string | Working directory | Current directory |","ref":"transport_options.html#configuration-options"},{"type":"extras","title":"Example: Running a Python MCP Server - Transport Options","doc":"```elixir\n{Hermes.Transport.STDIO, [\n  name: MyApp.MCPTransport,\n  client: MyApp.MCPClient, \n  command: \"python\",\n  args: [\"-m\", \"mcp.server\", \"my_server.py\"],\n  env: %{\"PYTHONPATH\" => \"/path/to/python/modules\"}\n]}\n```","ref":"transport_options.html#example-running-a-python-mcp-server"},{"type":"extras","title":"Example: Running a Node.js MCP Server - Transport Options","doc":"```elixir\n{Hermes.Transport.STDIO, [\n  name: MyApp.MCPTransport,\n  client: MyApp.MCPClient, \n  command: \"node\",\n  args: [\"server.js\"],\n  env: %{\"NODE_ENV\" => \"production\"}\n]}\n```","ref":"transport_options.html#example-running-a-node-js-mcp-server"},{"type":"extras","title":"HTTP/SSE Transport - Transport Options","doc":"Support for HTTP with Server-Sent Events (SSE) transport is planned for future releases. This will enable communication with remote MCP servers over HTTP.","ref":"transport_options.html#http-sse-transport"},{"type":"extras","title":"Configuration - Transport Options","doc":"```elixir\n{Hermes.Transport.HTTP, [\n  name: MyApp.HTTPTransport,\n  client: MyApp.MCPClient,\n  server: [\n    base_url: \"https://example.com\",\n    base_path: \"/mcp\", # defaults to \"/\"\n    sse_path: \"/sse\" # defaults to \"/sse\"\n  ],\n  headers: [{\"Authorization\", \"Bearer token\"}],\n  transport_opts: [verify: :verify_peer],\n  http_options: [recv_timeout: 30_000]\n]}\n```","ref":"transport_options.html#configuration"},{"type":"extras","title":"Configuration Options - Transport Options","doc":"| Option | Type | Description | Default |\n|--------|------|-------------|---------|\n| `:name` | atom | Registration name for the transport process | `__MODULE__` |\n| `:client` | atom or pid | The client process that will receive messages | Required |\n| `:server` | keyword | The SSE server config | Required |\n| `:server.base_url` | string | The SSE server base url | Required |\n| `:server.base_path` | string | The SSE server base path | `\"/\"`|\n| `:server.sse_path` | string | The SSE server base path for starting a SSE connection | `\"/sse\"`|\n| `:headers` | map | Additional request headers to be sent | `%{}` |\n| `:transport_opts` | keyword | Options to be passed to the underlying HTTP Client, yo ucan check the avaiable options on https://hexdocs.pm/mint/Mint.HTTP.html#connect/4-transport-options | System defaults |\n| `:http_options` | string | Options passed directly to the HTTP Client, you can check the available options on https://hexdocs.pm/finch/Finch.html#t:request_opt/0 | Current directory |","ref":"transport_options.html#configuration-options"},{"type":"extras","title":"Custom Transport Implementation - Transport Options","doc":"You can implement custom transports by creating a module that implements the `Hermes.Transport.Behaviour` behavior.","ref":"transport_options.html#custom-transport-implementation"},{"type":"extras","title":"Example: Custom TCP Transport - Transport Options","doc":"```elixir\ndefmodule MyApp.Transport.TCP do\n  @behaviour Hermes.Transport.Behaviour\n  use GenServer\n  \n  @impl true\n  def start_link(opts) do\n    GenServer.start_link(__MODULE__, opts, name: opts[:name])\n  end\n  \n  @impl true\n  def send_message(message) when is_binary(message) do\n    GenServer.call(__MODULE__, {:send, message})\n  end\n  \n  @impl true\n  def send_message(pid, message) when is_pid(pid) and is_binary(message) do\n    GenServer.call(pid, {:send, message})\n  end\n  \n  @impl true\n  def init(opts) do\n    client = opts[:client]\n    host = opts[:host] || \"localhost\"\n    port = opts[:port] || 8080\n    \n    case :gen_tcp.connect(String.to_charlist(host), port, [:binary, active: true]) do\n      {:ok, socket} ->\n        {:ok, %{socket: socket, client: client}}\n        \n      {:error, reason} ->\n        {:stop, reason}\n    end\n  end\n  \n  @impl true\n  def handle_call({:send, message}, _from, %{socket: socket} = state) do\n    case :gen_tcp.send(socket, message) do\n      :ok -> {:reply, :ok, state}\n      {:error, reason} -> {:reply, {:error, reason}, state}\n    end\n  end\n  \n  @impl true\n  def handle_info({:tcp, _socket, data}, %{client: client} = state) do\n    # Forward data to client\n    Process.send(client, {:response, data}, [:noconnect])\n    {:noreply, state}\n  end\n  \n  @impl true\n  def handle_info({:tcp_closed, _socket}, state) do\n    {:stop, :normal, state}\n  end\n  \n  @impl true\n  def handle_info({:tcp_error, _socket, reason}, state) do\n    {:stop, reason, state}\n  end\nend\n```","ref":"transport_options.html#example-custom-tcp-transport"},{"type":"extras","title":"Transport Selection Guidelines - Transport Options","doc":"When choosing a transport mechanism, consider:\n\n1. **Location**: For local servers, STDIO is typically the simplest and most efficient option.\n2. **Security**: For remote servers, HTTP/SSE with TLS provides secure communication.\n3. **Performance**: STDIO generally has lower latency for local communication.\n4. **Reliability**: HTTP transports can handle intermittent network issues more gracefully.","ref":"transport_options.html#transport-selection-guidelines"},{"type":"extras","title":"Transport Lifecycle Management - Transport Options","doc":"Transports in Hermes are designed to be supervised, with automatic recovery in case of failures:\n\n```elixir\n# In your supervision tree\nchildren = [\n  {Hermes.Transport.STDIO, transport_opts},\n  {Hermes.Client, client_opts},\n  # ...\n]\n\n# The transport will automatically restart if it crashes\nopts = [strategy: :one_for_one]\nSupervisor.start_link(children, opts)\n```","ref":"transport_options.html#transport-lifecycle-management"},{"type":"extras","title":"Message Handling","doc":"# Message Handling\n\nThis guide explains how Hermes MCP handles the flow of messages between clients and servers, including request/response patterns, notifications, and error handling.","ref":"message_handling.html"},{"type":"extras","title":"Message Types - Message Handling","doc":"Hermes implements all the standard JSON-RPC 2.0 message types as defined in the MCP specification:\n\n1. **Requests**: Messages with a method, parameters, and ID that expect a response\n2. **Responses**: Successful results or errors matching a specific request ID\n3. **Notifications**: One-way messages with no response expected","ref":"message_handling.html#message-types"},{"type":"extras","title":"Message Flow Architecture - Message Handling","doc":"```\n┌────────────┐      ┌────────────┐      ┌────────────┐\n│            │      │            │      │            │\n│   Client   │<─────│  Transport │<─────│   Server   │\n│            │─────>│            │─────>│            │\n└────────────┘      └────────────┘      └────────────┘\n```\n\n1. The client sends requests and notifications to the transport layer\n2. The transport delivers these messages to the server\n3. The server processes messages and sends responses or notifications\n4. The transport delivers these back to the client\n5. The client correlates responses with their original requests","ref":"message_handling.html#message-flow-architecture"},{"type":"extras","title":"Request/Response Correlation - Message Handling","doc":"Hermes automatically handles correlation between requests and responses using unique message IDs:\n\n```elixir\n# Inside Hermes.Client implementation\ndefp generate_request_id do\n  binary = < >\n\n  Base.url_encode64(binary)\nend\n```\n\nEach request is assigned a unique ID, and the client maintains a map of pending requests:\n\n```elixir\n# Simplified example of internal state\n%{\n  pending_requests: %{\n    \"request_id_123\" => {from, \"method_name\"},\n    \"request_id_456\" => {from, \"other_method\"}\n  }\n}\n```\n\nWhen a response arrives, it's matched to the original request and forwarded to the waiting process.","ref":"message_handling.html#request-response-correlation"},{"type":"extras","title":"Timeouts and Recovery - Message Handling","doc":"Hermes implements automatic timeout handling for requests:\n\n```elixir\n# Default timeout in milliseconds\n@default_timeout :timer.seconds(30)\n```\n\nIf a response isn't received within the timeout period, the client will:\n\n1. Remove the request from the pending requests map\n2. Send a cancellation notification to the server\n3. Return an error to the caller","ref":"message_handling.html#timeouts-and-recovery"},{"type":"extras","title":"Notification Handling - Message Handling","doc":"Notifications are one-way messages that don't expect a response. Hermes provides a simple API for sending notifications:\n\n```elixir\n# Internal implementation\ndefp send_notification(state, method, params \\\\ %{}) do\n  with {:ok, notification_data} <- encode_notification(method, params) do\n    send_to_transport(state.transport, notification_data)\n  end\nend\n```\n\nThe client sends the `notifications/initialized` notification after successful initialization.","ref":"message_handling.html#notification-handling"},{"type":"extras","title":"Protocol Message Encoding/Decoding - Message Handling","doc":"Hermes uses a structured approach to encode and validate protocol messages.\n\nThe `Hermes.Message` module provides robust validation of all messages against the MCP schema","ref":"message_handling.html#protocol-message-encoding-decoding"},{"type":"extras","title":"Error Handling Patterns - Message Handling","doc":"Hermes implements standardized error handling for all message types:","ref":"message_handling.html#error-handling-patterns"},{"type":"extras","title":"Protocol Errors - Message Handling","doc":"JSON-RPC protocol errors follow the standard error codes:\n\n```elixir\n# Example of handling an error response\ndefp handle_error(%{\"error\" => error, \"id\" => id}, id, state) do\n  {{from, _method}, pending} = Map.pop(state.pending_requests, id)\n\n  # Unblocks original caller\n  GenServer.reply(from, {:error, error})\n\n  %{state | pending_requests: pending}\nend\n```","ref":"message_handling.html#protocol-errors"},{"type":"extras","title":"Transport Errors - Message Handling","doc":"Transport-level errors are wrapped in a standardized format:\n\n```elixir\n# Example of handling a transport error\ndefp send_to_transport(transport, data) do\n  with {:error, reason} <- transport.send_message(data) do\n    {:error, {:transport_error, reason}}\n  end\nend\n```","ref":"message_handling.html#transport-errors"},{"type":"extras","title":"Application Errors - Message Handling","doc":"Application-level errors (e.g., in tool execution) are handled within the result structure:\n\n```elixir\n# Example result for a tool execution error\n{:error, %{\n  \"content\" => [%{\"type\" => \"text\", \"text\" => \"Error message\"}],\n  \"isError\" => true\n}}\n```","ref":"message_handling.html#application-errors"},{"type":"extras","title":"Rate Limiting Considerations - Message Handling","doc":"While Hermes does not impose rate limits directly, implementers should consider:\n\n1. Batching requests when appropriate\n2. Implementing backoff mechanisms for retries\n3. Monitoring request volumes in production","ref":"message_handling.html#rate-limiting-considerations"},{"type":"extras","title":"Security Considerations","doc":"# Security Considerations\n\nThis guide covers security best practices and implementation strategies for authentication and authorization with Hermes MCP.","ref":"security.html"},{"type":"extras","title":"MCP Security Overview - Security Considerations","doc":"The Model Context Protocol itself does not define specific authentication or authorization mechanisms. However, securing your MCP connections is essential, particularly when:\n\n1. Exposing MCP services over a network\n2. Handling sensitive data or operations\n3. Integrating with LLMs that have access to tools\n4. Operating in multi-user environments","ref":"security.html#mcp-security-overview"},{"type":"extras","title":"Transport Security - Security Considerations","doc":"","ref":"security.html#transport-security"},{"type":"extras","title":"STDIO Transport Security - Security Considerations","doc":"When using the STDIO transport for local processes:\n\n1. **Process Isolation**: Ensure the spawned process runs with appropriate permissions\n2. **Environment Variables**: Securely manage sensitive environment variables\n3. **Working Directory**: Set appropriate working directory permissions\n4. **Command Validation**: Validate the command and arguments before execution\n\nExample of secure STDIO transport configuration:\n\n```elixir\n# Secure STDIO transport configuration\n{Hermes.Transport.STDIO, [\n  name: MyApp.MCPTransport,\n  client: MyApp.MCPClient,\n  command: \"mcp\", # full path validation internally\n  args: [\"run\", path], # Validate path\n  env: %{\"SECRET_KEY\" => System.fetch_env!(\"MCP_SECRET_KEY\")},\n  cwd: Application.app_dir(:my_app, \"priv/mcp_server\")\n]}\n```","ref":"security.html#stdio-transport-security"},{"type":"extras","title":"HTTP/SSE Transport Security (Planned) - Security Considerations","doc":"For HTTP/SSE transports (planned for future releases):\n\n1. **TLS/SSL**: Always use HTTPS with proper certificate validation\n2. **CORS**: Configure appropriate Cross-Origin Resource Sharing policies\n3. **Rate Limiting**: Implement rate limiting to prevent abuse\n4. **Request Validation**: Validate all incoming requests\n\nExample of future HTTP transport configuration:\n\n```elixir\n# Future HTTP transport configuration\n{Hermes.Transport.HTTP, [\n  name: MyApp.HTTPTransport,\n  client: MyApp.MCPClient,\n  server: [base_url: \"https://example.com\", base_path: \"/mcp\"],\n  headers: [{\"Authorization\", \"Bearer #{token}\"}],\n  transport_opts: [\n    verify: :verify_peer,\n    cacertfile: CAStore.file_path(),\n    depth: 3,\n    verify_fun: :public_key.pkix_verify_hostname_match_fun(:https)\n  ]\n]}\n```","ref":"security.html#http-sse-transport-security-planned"},{"type":"extras","title":"Authentication Implementation Strategies - Security Considerations","doc":"While Hermes MCP doesn't include built-in authentication, here are several approaches to implement your own:","ref":"security.html#authentication-implementation-strategies"},{"type":"extras","title":"1. Transport-Level Authentication - Security Considerations","doc":"Add authentication at the transport level by including credentials in the transport configuration:\n\n```elixir\n# Example with future HTTP transport\n{Hermes.Transport.HTTP, [\n  # ... other options\n  headers: [\n    {\"Authorization\", \"Bearer #{token}\"},\n    {\"X-API-Key\", api_key}\n  ]\n]}\n```","ref":"security.html#1-transport-level-authentication"},{"type":"extras","title":"2. Initialization Authentication - Security Considerations","doc":"Extend the client capabilities to include authentication information during initialization:\n\n```elixir\n# Add authentication data to client initialization\nclient_capabilities = %{\n  \"resources\" => %{},\n  \"tools\" => %{},\n  \"experimental\" => %{\n    \"auth\" => %{\n      \"type\" => \"bearer\",\n      \"token\" => System.fetch_env!(\"MCP_AUTH_TOKEN\")\n    }\n  }\n}\n\n{Hermes.Client, [\n  # ... other options\n  capabilities: client_capabilities\n]}\n```\n\nThe server would then validate these credentials during the initialization handshake.","ref":"security.html#2-initialization-authentication"},{"type":"extras","title":"3. Custom Middleware Authentication - Security Considerations","doc":"Implement a custom authentication middleware that wraps the transport:\n\n```elixir\ndefmodule MyApp.Transport.AuthMiddleware do\n  @behaviour Hermes.Transport.Behaviour\n  \n  defstruct [:inner_transport, :auth_token]\n  \n  def new(inner_transport, auth_token) do\n    %__MODULE__{\n      inner_transport: inner_transport,\n      auth_token: auth_token\n    }\n  end\n  \n  @impl true\n  def start_link(opts) do\n    inner_transport = opts[:inner_transport]\n    auth_token = opts[:auth_token]\n    \n    with {:ok, transport} <- inner_transport.start_link(opts) do\n      {:ok, new(transport, auth_token)}\n    end\n  end\n  \n  @impl true\n  def send_message(%__MODULE__{} = transport, message) do\n    # Add authentication to outgoing JSON-RPC messages\n    case Jason.decode(message) do\n      {:ok, decoded} ->\n        authenticated = add_auth_meta(decoded, transport.auth_token)\n        case Jason.encode(authenticated) do\n          {:ok, json} -> transport.inner_transport.send_message(json)\n          error -> error\n        end\n      \n      error -> error\n    end\n  end\n  \n  defp add_auth_meta(%{\"method\" => _} = request, token) do\n    # Add auth metadata to the request\n    meta = %{\"_meta\" => %{\"auth\" => %{\"token\" => token}}}\n    request\n    |> Map.update(\"params\", meta, fn params -> Map.merge(params, meta) end)\n  end\n  \n  defp add_auth_meta(message, _token), do: message\nend\n```","ref":"security.html#3-custom-middleware-authentication"},{"type":"extras","title":"Authorization Implementation Strategies - Security Considerations","doc":"Authorization determines what operations a client can perform. Here are approaches to implement authorization:","ref":"security.html#authorization-implementation-strategies"},{"type":"extras","title":"1. Resource URI Validation - Security Considerations","doc":"Validate resource URIs against allowed patterns:\n\n```elixir\ndefmodule MyApp.ResourceValidator do\n  def validate_access(uri, user) do\n    case URI.parse(uri) do\n      %URI{scheme: \"file\"} = parsed ->\n        validate_file_access(parsed.path, user)\n        \n      %URI{scheme: \"db\"} ->\n        validate_db_access(uri, user)\n        \n      _ ->\n        {:error, :invalid_uri_scheme}\n    end\n  end\n  \n  defp validate_file_access(path, user) do\n    allowed_paths = user.allowed_paths\n    \n    if Enum.any?(allowed_paths, &String.starts_with?(path, &1)) do\n      :ok\n    else\n      {:error, :unauthorized_path}\n    end\n  end\n  \n  defp validate_db_access(uri, user) do\n    # Database-specific validation\n    # ...\n  end\nend\n```","ref":"security.html#1-resource-uri-validation"},{"type":"extras","title":"2. Tool Permission Management - Security Considerations","doc":"Implement permission checks for tool execution:\n\n```elixir\ndefmodule MyApp.ToolAuthorizer do\n  def authorize(tool_name, arguments, user) do\n    tool_permissions = user.tool_permissions || %{}\n    \n    case Map.get(tool_permissions, tool_name) do\n      nil ->\n        {:error, :tool_not_authorized}\n        \n      permissions ->\n        validate_arguments(tool_name, arguments, permissions)\n    end\n  end\n  \n  defp validate_arguments(\"file_write\", %{\"path\" => path}, permissions) do\n    allowed_paths = permissions.allowed_write_paths || []\n    \n    if Enum.any?(allowed_paths, &String.starts_with?(path, &1)) do\n      :ok\n    else\n      {:error, :path_not_authorized}\n    end\n  end\n  \n  # Additional validation rules for other tools\nend\n```","ref":"security.html#2-tool-permission-management"},{"type":"extras","title":"3. Capability-Based Security - Security Considerations","doc":"Implement capability-based security by only exposing specific capabilities to each client:\n\n```elixir\n# Create custom capabilities based on user permissions\ndefmodule MyApp.CapabilityGenerator do\n  def generate_for_user(user) do\n    %{\n      \"resources\" => generate_resource_capabilities(user),\n      \"tools\" => generate_tool_capabilities(user)\n    }\n  end\n  \n  defp generate_resource_capabilities(user) do\n    if user.can_access_resources do\n      %{\"subscribe\" => user.can_subscribe_resources}\n    else\n      nil\n    end\n  end\n  \n  defp generate_tool_capabilities(user) do\n    if length(user.allowed_tools) > 0 do\n      %{}\n    else\n      nil\n    end\n  end\nend\n```\n\nThen use these capabilities when initializing the client:\n\n```elixir\nuser = MyApp.Accounts.get_user!(user_id)\ncapabilities = MyApp.CapabilityGenerator.generate_for_user(user)\n\n{Hermes.Client, [\n  # ... other options\n  capabilities: capabilities\n]}\n```","ref":"security.html#3-capability-based-security"},{"type":"extras","title":"Best Practices for Secure MCP Implementation - Security Considerations","doc":"1. **Principle of Least Privilege**: Only grant the minimum necessary permissions\n2. **Input Validation**: Validate all user inputs, especially resource URIs and tool arguments\n3. **Audit Logging**: Log all MCP operations for security monitoring and compliance\n4. **Rate Limiting**: Implement rate limiting to prevent abuse\n5. **Secure Defaults**: Use secure defaults and require explicit opt-in for sensitive operations","ref":"security.html#best-practices-for-secure-mcp-implementation"},{"type":"extras","title":"Security with LLM Integration - Security Considerations","doc":"When integrating MCP with Language Models, additional considerations apply:\n\n1. **Tool Sandboxing**: Sandbox tool execution to prevent unintended consequences\n2. **Prompt Isolation**: Ensure prompts cannot access unauthorized information\n3. **User Confirmation**: Require user confirmation for sensitive operations\n4. **Context Boundaries**: Maintain strict boundaries between different LLM conversations\n5. **Output Filtering**: Filter LLM outputs to prevent sensitive data leakage","ref":"security.html#security-with-llm-integration"},{"type":"extras","title":"Troubleshooting","doc":"# Troubleshooting\n\nThis guide provides solutions for common issues encountered when working with Hermes MCP.","ref":"troubleshooting.html"},{"type":"extras","title":"Connection Issues - Troubleshooting","doc":"","ref":"troubleshooting.html#connection-issues"},{"type":"extras","title":"Client Cannot Connect to Server - Troubleshooting","doc":"**Symptoms:**\n- Client initialization fails\n- Timeouts during initialization\n- Transport errors\n\n**Potential Causes and Solutions:**\n\n1. **Server Not Running**\n   - Verify the server process is running\n   - Check process exit codes and logs\n   - Ensure the command path is correct\n\n   ```elixir\n   # Verify command can be found\n   if cmd = System.find_executable(\"mcp\") do\n     IO.puts(\"Found MCP at: #{cmd}\")\n   else\n     IO.puts(\"MCP command not found in PATH\")\n   end\n   ```\n\n2. **Transport Configuration**\n   - Double-check transport configuration parameters\n   - Verify working directory permissions\n   - Check for environment variable issues\n\n   ```elixir\n   # Test transport manually\n   {:ok, transport} = Hermes.Transport.STDIO.start_link(\n     client: self(),\n     command: \"mcp\",\n     args: [\"--version\"]\n   )\n   \n   # Should output version information\n   ```\n\n3. **Port Issues**\n   - Check for port conflicts in HTTP/SSE transport\n   - Verify firewall settings if connecting remotely\n   - Check timeout settings in your client configuration","ref":"troubleshooting.html#client-cannot-connect-to-server"},{"type":"extras","title":"Protocol Errors - Troubleshooting","doc":"","ref":"troubleshooting.html#protocol-errors"},{"type":"extras","title":"Version Negotiation Failures - Troubleshooting","doc":"**Symptoms:**\n- Error like \"Unsupported protocol version\"\n- Initialization fails after transport connects\n\n**Solutions:**\n- Verify both client and server support compatible protocol versions\n- Check the server logs for supported versions\n- Update your Hermes MCP client to a compatible version\n\n```elixir\n# Log initialization error for debugging\n# In your error handler\nLogger.error(\"Protocol version error: #{inspect(error)}\")\n```","ref":"troubleshooting.html#version-negotiation-failures"},{"type":"extras","title":"Capability Negotiation Issues - Troubleshooting","doc":"**Symptoms:**\n- Operations fail with errors about missing capabilities\n- Some features don't work as expected\n\n**Solutions:**\n- Check which capabilities are advertised by both client and server\n- Make sure required capabilities are included in your client configuration\n- Inspect server capabilities after initialization\n\n```elixir\n# Debug server capabilities\nserver_capabilities = Hermes.Client.get_server_capabilities(MyApp.MCPClient)\nIO.inspect(server_capabilities, label: \"Server capabilities\")\n```","ref":"troubleshooting.html#capability-negotiation-issues"},{"type":"extras","title":"Message Handling Issues - Troubleshooting","doc":"","ref":"troubleshooting.html#message-handling-issues"},{"type":"extras","title":"Request Timeouts - Troubleshooting","doc":"**Symptoms:**\n- Operations fail with timeout errors\n- Long-running operations consistently fail\n\n**Solutions:**\n- Increase the request timeout for specific operations\n- Check for performance issues on the server\n- Review network latency between client and server\n\n```elixir\n# Use a longer timeout for slow operations\n{:ok, result} = Hermes.Client.call_tool(\n  MyApp.MCPClient,\n  \"slow_operation\",\n  %{},\n  timeout: 120_000  # 2 minutes\n)\n```","ref":"troubleshooting.html#request-timeouts"},{"type":"extras","title":"Message Parsing Errors - Troubleshooting","doc":"**Symptoms:**\n- \"Failed to decode response\" errors\n- \"Invalid message\" errors\n\n**Solutions:**\n- Check for message format issues\n- Verify the server is sending valid JSON-RPC 2.0 messages\n- Enable verbose logging to inspect raw messages\n\n```elixir\n# Set debug logging for Hermes.Message\nLogger.configure(level: :debug)\n```","ref":"troubleshooting.html#message-parsing-errors"},{"type":"extras","title":"Resource-Related Issues - Troubleshooting","doc":"","ref":"troubleshooting.html#resource-related-issues"},{"type":"extras","title":"Resource Not Found - Troubleshooting","doc":"**Symptoms:**\n- `{:error, %{\"code\" => -32002, \"message\" => \"Resource not found\"}}`\n- Resource operations consistently fail\n\n**Solutions:**\n- Verify the resource URI is correct\n- Check if the server has access to the resource\n- List available resources to confirm what's accessible\n\n```elixir\n# List all available resources\n{:ok, %{\"resources\" => resources}} = Hermes.Client.list_resources(MyApp.MCPClient)\nIO.inspect(resources, label: \"Available resources\")\n```","ref":"troubleshooting.html#resource-not-found"},{"type":"extras","title":"Resource Content Issues - Troubleshooting","doc":"**Symptoms:**\n- Resource content is corrupted or truncated\n- Binary resources have encoding issues\n\n**Solutions:**\n- Check for encoding/decoding issues\n- Verify resource size limits\n- Try alternative resource access methods\n\n```elixir\n# Test reading a resource with explicit content type expectations\ncase Hermes.Client.read_resource(MyApp.MCPClient, \"file:///example.txt\") do\n  {:ok, %{\"contents\" => [%{\"text\" => text}]}} ->\n    IO.puts(\"Successfully read text content: #{String.slice(text, 0, 100)}...\")\n    \n  {:ok, %{\"contents\" => [%{\"blob\" => _blob}]}} ->\n    IO.puts(\"Successfully read binary content\")\n    \n  error ->\n    IO.inspect(error, label: \"Error reading resource\")\nend\n```","ref":"troubleshooting.html#resource-content-issues"},{"type":"extras","title":"Tool-Related Issues - Troubleshooting","doc":"","ref":"troubleshooting.html#tool-related-issues"},{"type":"extras","title":"Tool Not Found - Troubleshooting","doc":"**Symptoms:**\n- `{:error, %{\"code\" => -32601, \"message\" => \"Method not found\"}}`\n- Tool calls consistently fail\n\n**Solutions:**\n- Verify the tool name is correct\n- Check if the server has the tool available\n- List available tools to confirm what's accessible\n\n```elixir\n# List all available tools\n{:ok, %{\"tools\" => tools}} = Hermes.Client.list_tools(MyApp.MCPClient)\nIO.inspect(tools, label: \"Available tools\")\n```","ref":"troubleshooting.html#tool-not-found"},{"type":"extras","title":"Tool Execution Errors - Troubleshooting","doc":"**Symptoms:**\n- `{:ok, %{\"content\" => [...], \"isError\" => true}}`\n- Tool returns error responses\n\n**Solutions:**\n- Check tool arguments for correctness\n- Review server logs for detailed error information\n- Test with simplified arguments to isolate the issue\n\n```elixir\n# Step through tool execution with simple arguments\ncase Hermes.Client.call_tool(MyApp.MCPClient, \"problematic_tool\", %{\"simple\" => \"value\"}) do\n  {:ok, %{\"content\" => content, \"isError\" => true}} ->\n    IO.puts(\"Tool execution error:\")\n    Enum.each(content, fn\n      %{\"type\" => \"text\", \"text\" => text} -> IO.puts(\"  #{text}\")\n      _ -> nil\n    end)\n    \n  {:ok, result} ->\n    IO.puts(\"Tool executed successfully\")\n    \n  error ->\n    IO.inspect(error, label: \"Error calling tool\")\nend\n```","ref":"troubleshooting.html#tool-execution-errors"},{"type":"extras","title":"Prompt-Related Issues - Troubleshooting","doc":"","ref":"troubleshooting.html#prompt-related-issues"},{"type":"extras","title":"Prompt Template Errors - Troubleshooting","doc":"**Symptoms:**\n- Prompt retrieval fails with parameter errors\n- Unexpected prompt content\n\n**Solutions:**\n- Verify required arguments are provided\n- Check argument format and types\n- Inspect the prompt template requirements\n\n```elixir\n# List prompt arguments\n{:ok, %{\"prompts\" => prompts}} = Hermes.Client.list_prompts(MyApp.MCPClient)\nprompt = Enum.find(prompts, & &1[\"name\"] == \"target_prompt\")\nIO.inspect(prompt[\"arguments\"], label: \"Required arguments\")\n```","ref":"troubleshooting.html#prompt-template-errors"},{"type":"extras","title":"Supervision and Lifecycle Issues - Troubleshooting","doc":"","ref":"troubleshooting.html#supervision-and-lifecycle-issues"},{"type":"extras","title":"Process Crashes - Troubleshooting","doc":"**Symptoms:**\n- Client or transport processes terminate unexpectedly\n- Supervisor restarts processes frequently\n\n**Solutions:**\n- Review process crash reasons in logs\n- Check for resource leaks or memory issues\n- Consider adjusting supervision strategy\n\n```elixir\n# Check process state\ncase Process.whereis(MyApp.MCPClient) do\n  nil ->\n    IO.puts(\"Client process not running!\")\n    \n  pid ->\n    IO.puts(\"Client is running with PID: #{inspect(pid)}\")\n    Process.info(pid, :message_queue_len)\n    |> IO.inspect(label: \"Message queue length\")\nend\n```","ref":"troubleshooting.html#process-crashes"},{"type":"extras","title":"Memory Usage Issues - Troubleshooting","doc":"**Symptoms:**\n- Growing memory usage\n- Slow performance over time\n\n**Solutions:**\n- Check for message or state accumulation\n- Review resource cleanup procedures\n- Consider implementing periodic restarts for long-running processes","ref":"troubleshooting.html#memory-usage-issues"},{"type":"extras","title":"Debugging Techniques - Troubleshooting","doc":"","ref":"troubleshooting.html#debugging-techniques"},{"type":"extras","title":"Enabling Verbose Logging - Troubleshooting","doc":"Increase logging detail for better diagnostics:\n\n```elixir\n# In your config/dev.exs or at runtime\nLogger.configure(level: :debug)\n```","ref":"troubleshooting.html#enabling-verbose-logging"},{"type":"extras","title":"Inspecting Message Flow - Troubleshooting","doc":"To debug message flow between client and server:\n\n```elixir\n# Create a custom logger transport\ndefmodule MyApp.DebugTransport do\n  @behaviour Hermes.Transport.Behaviour\n  \n  defstruct [:inner_transport]\n  \n  def new(inner_transport) do\n    %__MODULE__{inner_transport: inner_transport}\n  end\n  \n  def start_link(opts) do\n    inner_transport_module = opts[:inner_transport_module]\n    inner_opts = Keyword.delete(opts, :inner_transport_module)\n    \n    with {:ok, transport} <- inner_transport_module.start_link(inner_opts) do\n      {:ok, new(transport)}\n    end\n  end\n  \n  def send_message(%__MODULE__{} = transport, message) do\n    Logger.debug(\">> OUTGOING: #{inspect(message)}\")\n    transport.inner_transport.send_message(message)\n  end\n  \n  def send_message(pid, message) when is_pid(pid) do\n    Logger.debug(\">> OUTGOING: #{inspect(message)}\")\n    transport = :sys.get_state(pid)\n    send_message(transport, message)\n  end\nend\n\n# Then use it in your setup\n{MyApp.DebugTransport, [\n  name: MyApp.DebugMCPTransport,\n  inner_transport_module: Hermes.Transport.STDIO,\n  client: MyApp.MCPClient,\n  command: \"mcp\",\n  args: [\"run\", \"server.py\"]\n]}\n```","ref":"troubleshooting.html#inspecting-message-flow"},{"type":"extras","title":"Testing Connection in Isolation - Troubleshooting","doc":"To test the transport separately:\n\n```elixir\n# In IEx console\niex> {:ok, transport} = Hermes.Transport.STDIO.start_link(\n...>   client: self(),\n...>   command: \"mcp\",\n...>   args: [\"run\", \"echo_server.py\"]\n...> )\niex> transport.send_message(\"{\\\"jsonrpc\\\":\\\"2.0\\\",\\\"method\\\":\\\"ping\\\",\\\"id\\\":1}\\n\")\niex> flush()  # Should see the response\n```","ref":"troubleshooting.html#testing-connection-in-isolation"},{"type":"extras","title":"Performance Considerations - Troubleshooting","doc":"","ref":"troubleshooting.html#performance-considerations"},{"type":"extras","title":"Optimizing Request Handling - Troubleshooting","doc":"For better performance:\n\n1. **Batch Requests When Possible**\n   - Group related operations together\n   - Minimize round-trips for small operations\n\n2. **Consider Resource Size**\n   - Be cautious with large resources\n   - Use pagination for large collections\n\n3. **Connection Management**\n   - Keep long-lived connections when appropriate\n   - Implement connection pooling for multi-user scenarios","ref":"troubleshooting.html#optimizing-request-handling"},{"type":"extras","title":"Getting Additional Help - Troubleshooting","doc":"If you're still experiencing issues:\n\n1. Check the [GitHub Issues](https://github.com/cloudwalk/hermes-mcp/issues) for similar problems\n2. Look for known issues in the [MCP Specification](https://spec.modelcontextprotocol.io/)\n3. Join the Elixir community channels to ask for help","ref":"troubleshooting.html#getting-additional-help"},{"type":"extras","title":"Examples","doc":"# Examples\n\nThis page provides practical examples of using Hermes MCP in different scenarios. Each example demonstrates a specific use case or pattern to help you implement MCP in your Elixir applications.","ref":"examples.html"},{"type":"extras","title":"Basic Client Example - Examples","doc":"This example shows a complete implementation of a simple Hermes MCP client that connects to a Python MCP server.\n\nGiven an Echo Python MCP server:\n\n```python\nfrom mcp.server.fastmcp import FastMCP\n\nmcp = FastMCP(\"Echo\")\n\n@mcp.resource(\"echo://{message}\")\ndef echo_resource(message: str) -> str:\n    \"\"\"Echo a message as a resource\"\"\"\n    return f\"Resource echo: {message}\"\n\n@mcp.tool()\ndef echo_tool(message: str) -> str:\n    \"\"\"Echo a message as a tool\"\"\"\n    return f\"Tool echo: {message}\"\n\n@mcp.prompt()\ndef echo_prompt(message: str) -> str:\n    \"\"\"Create an echo prompt\"\"\"\n    return f\"Please process this message: {message}\"\n\nif __name__ == \"__main__\":\n    mcp.run(transport='stdio')\n```\n\nYou can check the custom Mix.Tasks that `Hermes defines` to run and interactive with this server from Elixir, that are:\n\n- [stdio.echo](https://github.com/cloudwalk/hermes-mcp/blob/main/lib/mix/tasks/stdio.echo.ex)\n- [stdio.echo.interactive](https://github.com/cloudwalk/hermes-mcp/blob/main/lib/mix/tasks/stdio.echo.interactive.ex)","ref":"examples.html#basic-client-example"},{"type":"extras","title":"Authentication Example - Examples","doc":"This example demonstrates a custom authentication middleware for STDIO transport:\n\n```elixir\ndefmodule MyApp.AuthenticatedTransport do\n  @moduledoc \"\"\"\n  A transport middleware that adds authentication to JSON-RPC messages.\n  \"\"\"\n  \n  @behaviour Hermes.Transport.Behaviour\n  \n  alias Hermes.Transport.STDIO\n  \n  defstruct [:inner_transport, :auth_token, :client]\n  \n  @doc \"\"\"\n  Starts a new authenticated transport.","ref":"examples.html#authentication-example"},{"type":"extras","title":"Options - Examples","doc":"* `:inner_transport_module` - The underlying transport module to use (required)\n    * `:auth_token` - The authentication token to use (required)\n    * Other options are passed to the inner transport\n  \"\"\"\n  @impl true\n  def start_link(opts) do\n    client = Keyword.fetch!(opts, :client)\n    auth_token = Keyword.fetch!(opts, :auth_token)\n    inner_transport_module = Keyword.get(opts, :inner_transport_module, STDIO)\n    \n    # Remove our custom options before passing to inner transport\n    inner_opts = opts\n                 |> Keyword.delete(:auth_token)\n                 |> Keyword.delete(:inner_transport_module)\n    \n    with {:ok, transport} <- inner_transport_module.start_link(inner_opts) do\n      state = %__MODULE__{\n        inner_transport: transport,\n        auth_token: auth_token,\n        client: client\n      }\n      \n      pid = spawn_link(fn -> process_loop(state) end)\n      \n      # Register the process name if provided\n      if name = Keyword.get(opts, :name) do\n        Process.register(pid, name)\n      end\n      \n      {:ok, pid}\n    end\n  end\n  \n  @impl true\n  def send_message(pid, message) when is_pid(pid) and is_binary(message) do\n    send(pid, {:send, message})\n    :ok\n  end\n  \n  @impl true\n  def send_message(name, message) when is_atom(name) and is_binary(message) do\n    send(name, {:send, message})\n    :ok\n  end\n  \n  # Main process loop\n  defp process_loop(state) do\n    receive do\n      {:send, message} ->\n        # Add authentication to outgoing messages\n        case Jason.decode(message) do\n          {:ok, decoded} ->\n            authenticated = add_auth(decoded, state.auth_token)\n            \n            case Jason.encode(authenticated) do\n              {:ok, json} ->\n                Hermes.Transport.Behaviour.send_message(state.inner_transport, json <> \"\\n\")\n                \n              {:error, reason} ->\n                error = \"Failed to encode authenticated message: #{inspect(reason)}\"\n                IO.puts(:stderr, error)\n            end\n            \n          {:error, reason} ->\n            error = \"Failed to decode message for authentication: #{inspect(reason)}\"\n            IO.puts(:stderr, error)\n        end\n        \n      {:response, data} ->\n        # Forward responses directly to the client\n        send(state.client, {:response, data})\n        \n      other ->\n        IO.puts(:stderr, \"AuthenticatedTransport received unknown message: #{inspect(other)}\")\n    end\n    \n    process_loop(state)\n  end\n  \n  # Add authentication metadata to requests\n  defp add_auth(%{\"method\" => method, \"params\" => params} = request, token) do\n    # Skip authentication for the initialization message\n    if method == \"initialize\" do\n      request\n    else\n      auth_meta = %{\"_meta\" => %{\"auth\" => %{\"token\" => token}}}\n      \n      # Merge with existing params\n      updated_params = Map.merge(params || %{}, auth_meta)\n      %{request | \"params\" => updated_params}\n    end\n  end\n  \n  # Handle messages without params\n  defp add_auth(%{\"method\" => method} = request, token) do\n    if method == \"initialize\" do\n      request\n    else\n      auth_meta = %{\"_meta\" => %{\"auth\" => %{\"token\" => token}}}\n      Map.put(request, \"params\", auth_meta)\n    end\n  end\n  \n  # Pass through other message types unchanged\n  defp add_auth(message, _token), do: message\nend\n```\n\nUsage example with authenticated transport:\n\n```elixir\n# In your application's supervision tree\nchildren = [\n  # ...\n  \n  # Start the authenticated transport\n  {MyApp.AuthenticatedTransport, [\n    name: MyApp.MCPTransport,\n    client: MyApp.MCPClient,\n    auth_token: System.fetch_env!(\"MCP_AUTH_TOKEN\"),\n    inner_transport_module: Hermes.Transport.STDIO,\n    command: \"mcp\",\n    args: [\"run\", \"server.py\"]\n  ]},\n  \n  # Start the MCP client using the authenticated transport\n  {Hermes.Client, [\n    name: MyApp.MCPClient,\n    transport: MyApp.MCPTransport,\n    client_info: %{\n      \"name\" => \"MyAuthenticatedApp\",\n      \"version\" => \"1.0.0\"\n    },\n    capabilities: %{\n      \"resources\" => %{},\n      \"tools\" => %{},\n      \"prompts\" => %{}\n    }\n  ]},\n  \n  # ...\n]\n```","ref":"examples.html#options"},{"type":"extras","title":"Resource Abstraction Example - Examples","doc":"This example demonstrates a higher-level abstraction over MCP resources:\n\n```elixir\ndefmodule MyApp.Resource do\n  @moduledoc \"\"\"\n  A high-level abstraction for working with MCP resources.\n  \"\"\"\n  \n  alias Hermes.Client\n  \n  @type t :: %__MODULE__{\n    uri: String.t(),\n    name: String.t(),\n    description: String.t() | nil,\n    mime_type: String.t() | nil,\n    size: integer() | nil,\n    client: pid() | atom()\n  }\n  \n  defstruct [:uri, :name, :description, :mime_type, :size, :client]\n  \n  @doc \"\"\"\n  Lists all available resources from the MCP server.\n  \"\"\"\n  @spec list(pid() | atom()) :: {:ok, [t()]} | {:error, term()}\n  def list(client) do\n    case Client.list_resources(client) do\n      {:ok, %{\"resources\" => resources}} ->\n        resources = Enum.map(resources, fn resource ->\n          %__MODULE__{\n            uri: resource[\"uri\"],\n            name: resource[\"name\"],\n            description: resource[\"description\"],\n            mime_type: resource[\"mimeType\"],\n            size: resource[\"size\"],\n            client: client\n          }\n        end)\n        \n        {:ok, resources}\n        \n      error ->\n        error\n    end\n  end\n  \n  @doc \"\"\"\n  Reads the content of a resource.\n  \n  Returns the content as text if possible, otherwise as binary.\n  \"\"\"\n  @spec read(t() | String.t(), pid() | atom()) :: {:ok, String.t() | binary()} | {:error, term()}\n  def read(%__MODULE__{} = resource), do: read(resource.uri, resource.client)\n  def read(uri, client) when is_binary(uri) do\n    case Client.read_resource(client, uri) do\n      {:ok, %{\"contents\" => contents}} ->\n        content = case contents do\n          [%{\"text\" => text} | _] -> {:ok, text}\n          [%{\"blob\" => blob} | _] -> {:ok, blob}\n          [] -> {:error, :empty_content}\n          other -> {:error, {:unexpected_content, other}}\n        end\n        \n        content\n        \n      error ->\n        error\n    end\n  end\n  \n  @doc \"\"\"\n  Creates a resource from a map of resource attributes.\n  \"\"\"\n  @spec from_map(map(), pid() | atom()) :: t()\n  def from_map(map, client) do\n    %__MODULE__{\n      uri: map[\"uri\"],\n      name: map[\"name\"],\n      description: map[\"description\"],\n      mime_type: map[\"mimeType\"],\n      size: map[\"size\"],\n      client: client\n    }\n  end\n  \n  @doc \"\"\"\n  Finds a resource by name.\n  \"\"\"\n  @spec find_by_name(String.t(), pid() | atom()) :: {:ok, t()} | {:error, :not_found} | {:error, term()}\n  def find_by_name(name, client) do\n    with {:ok, resources} <- list(client) do\n      case Enum.find(resources, fn r -> r.name == name end) do\n        nil -> {:error, :not_found}\n        resource -> {:ok, resource}\n      end\n    end\n  end\n  \n  @doc \"\"\"\n  Checks if a resource is text-based or binary.\n  \"\"\"\n  @spec text?(t()) :: boolean()\n  def text?(%__MODULE__{mime_type: mime_type}) when is_binary(mime_type) do\n    String.starts_with?(mime_type, \"text/\") or\n      mime_type in [\"application/json\", \"application/xml\", \"application/javascript\"]\n  end\n  def text?(_), do: false\nend\n```\n\nUsage example:\n\n```elixir\n# List all resources\n{:ok, resources} = MyApp.Resource.list(MyApp.MCPClient)\n\n# Find a specific resource by name\n{:ok, config} = MyApp.Resource.find_by_name(\"config.json\", MyApp.MCPClient)\n\n# Read a resource's content\n{:ok, content} = MyApp.Resource.read(config)\n\nif MyApp.Resource.text?(config) do\n  IO.puts(\"Text content: #{content}\")\nelse\n  IO.puts(\"Binary content: #{byte_size(content)} bytes\")\nend\n```\n\nThese examples demonstrate different approaches to using Hermes MCP in your Elixir applications. You can adapt and extend these patterns to suit your specific needs.","ref":"examples.html#resource-abstraction-example"},{"type":"extras","title":"RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"# RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","ref":"rfc.html"},{"type":"extras","title":"1. Introduction - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"This document defines the design and architecture of an Elixir library that implements both the client and server aspects of the Model Context Protocol (MCP). The goal is to provide a unified solution that supports robust JSON‑RPC 2.0 communication, leverages Elixir's concurrency and fault tolerance, and integrates seamlessly with Phoenix for server operations. The library will expose a high‑level client API that abstracts transport details and a server API that is compliant with Phoenix conventions for building HTTP endpoints. The design is modular, extensible, and structured to support future enhancements.\n\nFor further context on MCP, see the [Anthropic announcement](https://www.anthropic.com/news/model-context-protocol) and the [Glama Quickstart](https://glama.ai/blog/2024-11-25-model-context-protocol-quickstart).\n\n---","ref":"rfc.html#1-introduction"},{"type":"extras","title":"2. Library Architecture Overview - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"","ref":"rfc.html#2-library-architecture-overview"},{"type":"extras","title":"2.1 General Structure - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The library is divided into two main components: the Client and the Server. Both components share core infrastructure for JSON‑RPC handling, configuration, and error management. The Client is designed to provide a high‑level API that abstracts connection management and messaging details, while the Server exposes MCP functionality through Phoenix‑compliant endpoints.\n\nA central configuration module governs the behavior of both components, allowing runtime customization of timeouts, protocol versions, and transport preferences. Middleware hooks are provided at strategic points to support additional processing such as authentication, logging, or transformation.","ref":"rfc.html#2-1-general-structure"},{"type":"extras","title":"2.2 Core Components - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"At the lowest level, a transport abstraction layer decouples the specifics of STDIO, HTTP, or other transports from the core logic. The connection manager (implemented as a GenServer) oversees the client's lifecycle, including the initialization handshake and message correlation. For the server, a dispatcher module routes incoming JSON‑RPC messages (received via a Phoenix controller or Plug) to the appropriate tool handler registered in a tool registry. These components are organized under a comprehensive supervision tree to ensure fault tolerance and automatic recovery.\n\n---","ref":"rfc.html#2-2-core-components"},{"type":"extras","title":"3. Protocol Lifecycle Management - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The Model Context Protocol defines specific lifecycle requirements that both clients and servers must implement. Our Elixir library provides unified lifecycle management that serves both roles through shared infrastructure while maintaining clear separation of concerns.","ref":"rfc.html#3-protocol-lifecycle-management"},{"type":"extras","title":"3.1 Shared Infrastructure - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The library implements the protocol lifecycle through a state machine that governs connection establishment, message exchange, and termination. This shared infrastructure leverages Elixir's GenServer and supervision capabilities to provide robust state management and fault tolerance regardless of whether the library is operating in client or server mode.\n\n```mermaid\nstateDiagram-v2\n    [*] --> Initializing\n    Initializing --> Active: Capability Exchange Complete\n    Active --> Recovering: Connection Issue\n    Recovering --> Initializing: Retry Connection\n    Active --> Terminating: Shutdown Requested\n    Recovering --> Terminating: Recovery Failed\n    Terminating --> [*]\n```","ref":"rfc.html#3-1-shared-infrastructure"},{"type":"extras","title":"3.2 Client Lifecycle - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"When operating as a client, the library actively initiates and manages connections to MCP servers. The client connection manager handles the following responsibilities:\n\nThe client begins by establishing a transport connection and sending an initialization request containing its capabilities and protocol version. Upon receiving the server's capability response, it validates compatibility and transitions to the active state. Throughout the session, it maintains correlation between requests and responses while monitoring connection health.\n\nDuring recovery scenarios, the client connection manager attempts to reestablish the transport connection and reinitialize the protocol session. It provides configurable retry policies and transparent recovery when possible.","ref":"rfc.html#3-2-client-lifecycle"},{"type":"extras","title":"3.3 Server Lifecycle - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"As a server, the library acts as a passive participant in lifecycle management, responding to client initiatives while maintaining session isolation. The server session manager creates dedicated processes for each client connection, ensuring proper resource cleanup on termination.\n\nWhen receiving a client's initialization request, the server validates the protocol version compatibility and responds with its supported capabilities. It then maintains session state for the connection, tracking active subscriptions and managing resource access. The server uses Phoenix's connection management facilities when operating over HTTP/SSE transport.","ref":"rfc.html#3-3-server-lifecycle"},{"type":"extras","title":"3.4 Lifecycle Integration - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The following sequence diagram illustrates how the client and server components interact during a typical protocol lifecycle:\n\n```mermaid\nsequenceDiagram\n    participant Client\n    participant Transport\n    participant Server\n    \n    Note over Client,Server: Transport Establishment\n    Client->>Transport: Connect\n    Transport->>Server: Accept Connection\n    \n    Note over Client,Server: Initialization Phase\n    Client->>Server: Initialize Request (capabilities)\n    Server-->>Client: Initialize Response (capabilities)\n    Client->>Server: Initialized Notification\n    \n    Note over Client,Server: Active Session\n    Client->>Server: Protocol Messages\n    Server-->>Client: Protocol Responses\n    \n    Note over Client,Server: Termination\n    Client->>Server: Close Connection\n    Server-->>Client: Acknowledge Close\n```","ref":"rfc.html#3-4-lifecycle-integration"},{"type":"extras","title":"3.5 Error Management - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The library implements comprehensive error handling for both client and server roles. Errors are categorized into:\n\nProtocol Errors: Version mismatches or capability incompatibilities are handled through standardized error responses that maintain protocol compliance.\n\nTransport Errors: Connection failures trigger appropriate recovery mechanisms based on the component's role. Clients attempt reconnection while servers clean up affected sessions.\n\nApplication Errors: Errors during tool execution or resource access are propagated through the protocol's error reporting mechanisms while maintaining session stability.\n\nThis unified approach to lifecycle management ensures protocol compliance while providing robust operation in both client and server roles. The library's supervision strategies and process isolation enable reliable recovery from failure scenarios while maintaining proper resource cleanup.","ref":"rfc.html#3-5-error-management"},{"type":"extras","title":"4. Engineering Considerations - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"","ref":"rfc.html#4-engineering-considerations"},{"type":"extras","title":"4.1 Client Implementation - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The client's core responsibility is to manage the connection and message exchange with an MCP server. A GenServer will serve as the connection manager, maintaining state (such as pending message IDs and associated callbacks) and handling the initialization handshake. The transport layer will be defined by a set of behaviors, which will allow developers to implement additional transport modules (e.g., STDIO or WebSocket) without modifying the core client logic. Error handling is performed through standardized mappings to Elixir's error tuples, and configurable timeouts are implemented to avoid blocking calls.","ref":"rfc.html#4-1-client-implementation"},{"type":"extras","title":"4.2 Server Implementation - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"For the server, Phoenix integration is a primary design goal. The library will provide a Plug/Phoenix module that acts as the HTTP interface for MCP. Incoming JSON‑RPC messages are first parsed and validated, then dispatched to the correct tool handler as registered in the tool registry. The dispatcher module uses asynchronous Tasks to invoke tool handlers, ensuring that long‑running operations do not delay the overall request-response cycle. Session management is achieved by isolating client interactions within dedicated processes, which allows the server to maintain state and context for each connection independently. Additionally, extensive logging and observability hooks are built in to facilitate debugging and performance monitoring.","ref":"rfc.html#4-2-server-implementation"},{"type":"extras","title":"4.3 Shared and Extensible Architecture - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"Both the client and server components share a common configuration module that reads settings from Elixir's configuration files (e.g., config.exs). Middleware hooks are available for both sides, enabling developers to insert custom authentication, logging, or transformation logic into the processing pipeline. The design adheres to a strict supervision tree structure to automatically recover from process failures and to maintain high availability. This shared architecture not only simplifies maintenance but also ensures that future enhancements can be integrated with minimal disruption.","ref":"rfc.html#4-3-shared-and-extensible-architecture"},{"type":"extras","title":"5. Roadmap for Version 0.1.0 - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The roadmap for version 0.1.0 focuses on delivering a robust and well-tested client interface before expanding to full server functionality.","ref":"rfc.html#5-roadmap-for-version-0-1-0"},{"type":"extras","title":"5.1 Phase 1: Design and Requirements (Weeks 1-2) - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"During this phase, a detailed design document is created based on the MCP specification. This includes defining the minimal set of features for the client, specifying the public API functions (connect/1, request/2, notify/2), and designing the transport abstraction layer. Prototypes for the connection manager are built and validated against simulated MCP server responses.","ref":"rfc.html#5-1-phase-1-design-and-requirements-weeks-1-2"},{"type":"extras","title":"5.2 Phase 2: Core Client Module Implementation (Weeks 3-5) - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"In this phase, the core client module is developed. The connection manager GenServer is implemented, handling the JSON‑RPC handshake and managing state for pending requests. Basic error handling and timeout logic are integrated, and unit tests are written for each component. The transport abstraction is implemented initially for STDIO, with a clear interface for future extensions.","ref":"rfc.html#5-2-phase-2-core-client-module-implementation-weeks-3-5"},{"type":"extras","title":"5.3 Phase 3: Transport Abstraction and High-Level API (Weeks 6-7) - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The focus shifts to refining the transport abstraction layer, ensuring that it supports multiple transport mechanisms seamlessly. The high‑level client API is finalized to provide idiomatic functions that hide low‑level details. Documentation is updated with usage examples and configuration guidelines.","ref":"rfc.html#5-3-phase-3-transport-abstraction-and-high-level-api-weeks-6-7"},{"type":"extras","title":"5.4 Phase 4: Integration Testing and Documentation (Weeks 8-9) - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"Comprehensive integration tests are performed by simulating connections to a basic test MCP server. Detailed documentation, including API reference guides and example applications, is produced. Performance metrics, logging outputs, and error scenarios are verified to ensure that the client component meets production standards.","ref":"rfc.html#5-4-phase-4-integration-testing-and-documentation-weeks-8-9"},{"type":"extras","title":"5.5 Phase 5: Beta Release and Feedback (Week 10) - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"A beta version of version 0.1.0 is released to early adopters. Feedback is collected regarding usability, stability, and performance. Based on this feedback, necessary adjustments are made before finalizing the release. This phase sets the stage for subsequent versions that will incorporate full server functionality with Phoenix integration and additional transport support.","ref":"rfc.html#5-5-phase-5-beta-release-and-feedback-week-10"},{"type":"extras","title":"6. Conclusion - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"This RFC has outlined the complete technical design for an Elixir MCP library that provides both client and server capabilities. The public API is designed to be high‑level for the client—abstracting connection and message handling—and fully compliant with Phoenix for the server side. The document details the architecture, internal flows, and the phased roadmap for version 0.1.0, focusing initially on establishing a robust client interface. The modular, extensible design ensures that the library can evolve to support additional features such as dynamic tool registration, middleware enhancements, and alternative transport mechanisms.","ref":"rfc.html#6-conclusion"},{"type":"extras","title":"References - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"1.\tIntroducing the Model Context Protocol – Anthropic\n2.\tMCP Quickstart – Glama\n3.\tModel Context Protocol – Linkup","ref":"rfc.html#references"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}